using System;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using NodaTime;

namespace DysonNetwork.Common.Models;

/// <summary>
/// Represents a connection between an account and an authentication provider
/// </summary>
public class AccountConnection
{
    /// <summary>
    /// Unique identifier for the connection
    /// </summary>
    [Key]
    [DatabaseGenerated(DatabaseGeneratedOption.Identity)]
    public string Id { get; set; } = null!;

    /// <summary>
    /// The account ID this connection is associated with
    /// </summary>
    public string? AccountId { get; set; }

    /// <summary>
    /// The authentication provider (e.g., "google", "github")
    /// </summary>
    [Required]
    [MaxLength(50)]
    public string Provider { get; set; } = null!;

    /// <summary>
    /// The unique identifier for the user from the provider
    /// </summary>
    [Required]
    [MaxLength(256)]
    public string ProvidedIdentifier { get; set; } = null!;

    /// <summary>
    /// Display name for the connection
    /// </summary>
    [MaxLength(100)]
    public string? DisplayName { get; set; }

    /// <summary>
    /// OAuth access token from the provider
    /// </summary>
    public string? AccessToken { get; set; }

    /// <summary>
    /// OAuth refresh token from the provider (if available)
    /// </summary>
    public string? RefreshToken { get; set; }

    /// <summary>
    /// When the access token expires (if available)
    /// </summary>
    public Instant? ExpiresAt { get; set; }

    /// <summary>
    /// When the connection was first established
    /// </summary>
    public Instant CreatedAt { get; set; }

    /// <summary>
    /// When the connection was last used
    /// </summary>
    public Instant? LastUsedAt { get; set; }

    /// <summary>
    /// Additional metadata about the connection
    /// </summary>
    [Column(TypeName = "jsonb")]
    public Dictionary<string, object>? Meta { get; set; }

    /// <summary>
    /// Navigation property for the associated account
    /// </summary>
    [ForeignKey(nameof(AccountId))]
    public virtual Account? Account { get; set; }
}
